/***************************************************************************
 *   Copyright (C) 2007 by Mauro Rodrigues				   *
 *   mauro.r@ua.pt							   *
 *                                                                         *
 *   PhUA - Projecto Humanóide da Universidade de Aveiro 07/08		   *
 *	@Sistemas de Tempo-Real 07/08					   *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <linux/types.h>
#include <math.h>

#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#include <sched.h>

#include <pman.h>
#include <pmandefs.h>
#include "defs.h"
#include <sem_utils.h>
#include <SerialPort.h>
#include <message.h>

//--------------------------
// Variaveis para o pan tilt
//--------------------------
signed char ang_max_tilt=20;
signed char ang_min_tilt=-40;

signed char ang_max_pan=90;
signed char ang_min_pan=-90;
//--------------------------

int end=0;

void sighand(int sig)
{
  if(sig==SIGINT)
    end=1;
  return;
}

/*
 *  Main
 */
int main(int argc, char *argv[])
{
/* **********************
 *	PMAN stuff 	*
 ************************/
	int pschedstat;       // Op status 
	char *pname = "ctrl";// Process name (must be unique)
	key_t sem_psched_key,shmem_psched_key;
	int actcnt=0;         // Activation counter
	struct timeval ta;

	/* Signal management */
	sigset_t set;
	struct sigaction act;
	/* Signal management set-up*/
	/* Sig mask with every signal but INT and PMAN_ACTIVATE_SIG */
	sigemptyset(&set);
	sigfillset(&set);
	sigdelset(&set,PMAN_ACTIVATE_SIG);
	sigdelset(&set,SIGINT);
	/* Signal handler */
	act.sa_handler=sighand;
	sigemptyset(&act.sa_mask);
	act.sa_flags=0;
	sigaction(PMAN_ACTIVATE_SIG,&act,0);
	sigaction(SIGINT,&act,0);
	/*
	* Initialize PMAN lib
	*/
	shmem_psched_key = SHMEM_PMAN_KEY;
	sem_psched_key   = SEM_PMAN_KEY;
	/* PMAN initializations (attach to existing process table)*/ 
	if((pschedstat = PMAN_init(SHMEM_PMAN_KEY, SEM_PMAN_KEY, NULL, 0,PMAN_ATTACH)))
	{
		fprintf(stderr, "\n [%s]: PMAN_init (PMAN_ATTACH) failed (return code %d)\n",pname, pschedstat);
		exit(EXIT_FAILURE);
	}
	/* Fill PID in process table */ 
	pschedstat=PMAN_attach(pname,getpid());
	if(pschedstat) 
	{
		fprintf(stderr, "\n [%s]: PMAN_attach failed (return code %d)\n",pname,pschedstat);
		exit(EXIT_FAILURE);
	}
	/*
	 * Variables
	 */
	Message port;
	int sstat, sem_rtdb_id;
	signed char servos[3];//val[3]={0, 0, 0};
	key_t sem_rtdb_key, shmem_rtdb_key;
	union dsemun sem_union;
	sem_rtdb_key = SEM_RTDB_KEY;
	shmem_rtdb_key = SHMEM_RTDB_KEY;

/******************************
 * Retrieve semaphore RTDB ID *
 ******************************/
	sem_rtdb_id = semget((key_t)SEM_RTDB_KEY, 1, 0666);
	if(sem_rtdb_id == -1) 
		return -2;

/****************************************
 * Shared Memory for Real Time DataBase *
 ****************************************/
	void *shm_rtdb = (void *)0;
	struct RTDB *sh_rtdb;
	int shm_rtdb_id;

	shm_rtdb_id = shmget(shmem_rtdb_key, sizeof(RTDB), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_rtdb_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_RTDB on obj_track!\n");
		exit(EXIT_FAILURE);
	}
	shm_rtdb = shmat(shm_rtdb_id, (void *)0, 0);
	if(shm_rtdb == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_RTDB on obj_track!\n");
		exit(EXIT_FAILURE);
	}

	sh_rtdb = (RTDB *)shm_rtdb;

/****************************************
 * Serial Communication Initializations *
 ****************************************/

	//seleccionar o controlo do servo da cabeça e inicialização dos seus parametros
	for(unsigned i=0;i<3;i++)
	{
		servos[i]=0;
	}
	servos[0]=1;
	port.applyjoint(HEAD, PARAM_SPECIAL, servos);
	for(unsigned i=0;i<3;i++)
	{
		servos[i]=10;
	}
	port.applyjoint(HEAD, PARAM_VELOCITY, servos);

	/*
	* Control loop
	*/
	while(!end)
	{

		/* Wait for activation instant */
		sigsuspend(&set);

		PMAN_trace_evt_set(pname,'S');

		gettimeofday(&ta,NULL);

		printf("\n   [%s Starting time %d, time:%6ld:%6ld]", argv[0],actcnt++,ta.tv_sec, ta.tv_usec);

		{
		// Instruções de controlo AQUI!
		}

		/* Critical Region - Access to RTDB */
		sem_wait(sem_rtdb_id);
		// update RTDB
		sem_signal(sem_rtdb_id);

		PMAN_epilogue(pname); // Mark process as finished
	}

	/*
	* Release resources before exit
	*/

	// Release & Delete RTDB Shared Memory
	if(shmdt(shm_rtdb) == -1)
	{
		fprintf(stderr, "shmdt failed dettaching SHM_RTDB on ctrl!\n");
		exit(EXIT_FAILURE);
	}
	if(shmctl(shm_rtdb_id, IPC_RMID, 0) == -1)
	{
		fprintf(stderr, "shmctl failed deleting SHM_RTDB on ctrl!\n");
		exit(EXIT_FAILURE);
	}

	/* Deletes semaphore RTDB */
	sstat=semctl(sem_rtdb_id, 0, IPC_RMID, sem_union);
	if(sstat == -1)
	{
		fprintf(stderr,"semctl (IPC_RMID) failed while deleting SEM_RTDB!\n");
		return -3;
	}

	PMAN_close(PMAN_CLLEAVE); // Release PMAN resources

	printf("\nCleanup complete!\n");
	exit(EXIT_SUCCESS);
 
}
