/***************************************************************************
 *   Copyright (C) 2007 by Mauro Rodrigues				   *
 *   mauro.r@ua.pt							   *
 *                                                                         *
 *   PhUA - Projecto Humanóide da Universidade de Aveiro 07/08		   *
 *	@Sistemas de Tempo-Real 07/08					   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>

#include <cvision.h>
#include <visiondefs.h>

#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/mman.h>
#include <linux/types.h>
#include <math.h>
#include <signal.h>

#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#include <sched.h>

#include <pmandefs.h>
#include <sem_utils.h>
#include <visiondefs.h>
#include <pman.h>

using namespace std;

int device = 0;

int end=0;

/*
 * Function prototypes
 */ 
static void endme(int dummy);
//int linux_sched_fifo(int pindex);


/*
 * main
 */
int main(int argc, char *argv[])
{
/* **********************
 *	PMAN stuff 	*
 ************************/
	int pschedstat, proc_priority;
	key_t sem_psched_key, shmem_psched_key;
	/*
	* Initializations
	*/
	signal(SIGINT, endme);   /* Set signal handler */
	/* PMAN initializations */
	shmem_psched_key = SHMEM_PMAN_KEY; /* PMan shmem & sem keys */
	sem_psched_key   = SEM_PMAN_KEY;

	if((pschedstat = PMAN_init(shmem_psched_key, sem_psched_key, (void*)linux_sched_fifo, sizeof(proc_priority),PMAN_NEW)))
	{
		fprintf(stderr, "\n [%s] PMAN_init failed (return code %d)\n",argv[0],pschedstat);
		exit(EXIT_FAILURE);
	}
	/* Add process data to PTable */
	/* Image Processing */
	proc_priority=30;
	PMAN_procadd("im_proc",PMAN_NOPID,1,0,100000,&proc_priority,sizeof(proc_priority));
	/* Object Tracking */
	proc_priority=40;
	PMAN_procadd("obj_track",PMAN_NOPID,1,1,100000,&proc_priority,sizeof(proc_priority));
	/* Control */
	proc_priority=50;
	PMAN_procadd("ctrl",PMAN_NOPID,1,0,200000,&proc_priority,sizeof(proc_priority));
	/* Defining precedences */
	PMAN_prec_add("obj_track","ctrl");
	PMAN_prec_add("im_proc","ctrl");
	PMAN_prec_add("im_proc","obj_track");

	PMAN_print_prec();

	/*
	 * Variables
	 */
	int sstat, sem_atop_id;
	CVision obj1(Gwidth, Gheight, device, Gdepth, Gchannels);
	key_t sem_atop_key, shmem_atop_key;
	union dsemun sem_union;
	sem_atop_key = SEM_ATOP_KEY;
	shmem_atop_key = SHMEM_ATOP_KEY;

/**********************************
 * Create and init semaphore ATOP *
 **********************************/
	sem_atop_id = semget((key_t)SEM_ATOP_KEY, 1, 0666 | IPC_CREAT);
	if(sem_atop_id == -1)
		return -2;

	sem_union.val=1;
	sstat=semctl(sem_atop_id, 0, SETVAL, sem_union);
	if(sstat == -1) 
		return -2;

/************************************************************
 * Shared Memory for Acquisition to Processing Comunication *
 ************************************************************/
	void *shm_atop = (void *)0;
	CVision *sh_atop;
	int shm_atop_id;

	shm_atop_id = shmget(shmem_atop_key, sizeof(CVision), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_atop_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_ATOP on im_acq!\n");
		exit(EXIT_FAILURE);
	}
	shm_atop = shmat(shm_atop_id, (void *)0, 0);
	if(shm_atop == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_ATOP on im_acq!\n");
		exit(EXIT_FAILURE);
	}

	sh_atop = (CVision *)shm_atop;

/************************
 *	 Main loop 	*
 ************************/
	while( !end )
	{
		/* Capture New Image */
		obj1.Refresh();
		//PMAN_print();

		/* Critical Region */
		sem_wait(sem_atop_id);
		*sh_atop = obj1;
		sem_signal(sem_atop_id);

		/* Activates processes */
		PMAN_tick();

		printf("[%s] tick...\n", argv[0]);
	}
	/* Save trace data */
	PMAN_trace_save();

/**********************
 * Release resourcess *
 **********************/
	if(shmdt(shm_atop) == -1)// Release AtoP Shared Memory
	{
		fprintf(stderr, "shmdt failed dettaching SHM_ATOP on im_acq!\n");
		exit(EXIT_FAILURE);
	}

	PMAN_close(PMAN_CLFREE); // Release PMAN resources 

	printf("[%s] Cleanup OK!!\n",argv[0]);

	return 0;
}

/* ------------------------------------------------------------------------- */
/*
 * SIGINT signal handler
 */
static void endme(int dummy)
{
  end = 1;
}

#if 0
/*
 * QoS function
 */
int linux_sched_fifo(int pindex)
{
  struct sched_param proc_sched;
  int prio;
  pid_t pid;
  PROC_TYPE procdata;
  void * qosdata_ptr;

  /* Get QoS data */  
  procdata=p_table->proc[pindex];
  qosdata_ptr = (void *)((int)p_table + procdata.PROC_qosdata); // Generic code. Get pointer to beginning of QoS data area
  prio = *((int *)qosdata_ptr); // QoS policy specific code! Assign correct pointer type
  pid=procdata.PROC_id;
  
  printf("\n<QoS, pindex=%d pid=%d prio=%d>",pindex, pid, prio);

  /* Set the process priority and scheduling policy */
  proc_sched.sched_priority=prio;
  if( sched_setscheduler(pid,SCHED_FIFO,&proc_sched) == -1)
    {
      fprintf(stderr,"\n [QoS]: Error setting the priority (sched_setscheduler)!");
      return -3;
    }
  
  return 0;
}
#endif
