/***************************************************************************
 *   Copyright (C) 2007 by Mauro Rodrigues				   *
 *   mauro.r@ua.pt							   *
 *                                                                         *
 *   PhUA - Projecto Humanóide da Universidade de Aveiro 07/08		   *
 *	@Sistemas de Tempo-Real 07/08					   *
 ***************************************************************************/

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <linux/types.h>
#include <math.h>
#include <pthread.h>

#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#include <sched.h>

#include <highgui.h>
#include <cv.h>
#include <cxcore.h>
#include <cvaux.h>

#include <pman.h>
#include <cvision.h>
#include <visiondefs.h>
#include <pmandefs.h>
#include <sem_utils.h>
#include "defs.h"

int end=0;

using namespace std;

/* Prototypes */
void sighand(int sig);

/*
 *  Main
 */
int main(int argc, char *argv[]) 
{
/* **********************
 *	PMAN stuff 	*
 ************************/
	int pschedstat;            // Op status
	char *pname = "im_proc";// Process name (must be unique)
	key_t sem_psched_key,shmem_psched_key;
	int actcnt=0;         // Processe activation counter
	struct timeval ta;
	/* Signal management */
	sigset_t set;
	struct sigaction act;
	/*
	* Initializations
	*/
	/* Signal management set-up*/
	/* Sig mask with every signal but INT and PMAN_ACTIVATE_SIG */
	sigemptyset(&set);
	sigfillset(&set);
	sigdelset(&set,PMAN_ACTIVATE_SIG);
	sigdelset(&set,SIGINT);
	/* Signal handler */
	act.sa_handler=sighand;
	sigemptyset(&act.sa_mask);
	act.sa_flags=0;
	sigaction(PMAN_ACTIVATE_SIG,&act,0);
	sigaction(SIGINT,&act,0);
	/*
	* Initialize PMAN lib
	*/
	shmem_psched_key = SHMEM_PMAN_KEY;
	sem_psched_key   = SEM_PMAN_KEY;
	/* PMAN initializations (attach to existing process table)*/
	if((pschedstat = PMAN_init(SHMEM_PMAN_KEY, SEM_PMAN_KEY, NULL, 0,PMAN_ATTACH)))
	{
		fprintf(stderr, "\n [%s]: PMAN_init (PMAN_ATTACH) failed (return code %d)\n",pname, pschedstat);
		exit(EXIT_FAILURE);
	}
	/* Fill PID in process table */
	pschedstat=PMAN_attach(pname,getpid());
	if(pschedstat)
	{
		fprintf(stderr, "\n [%s]: PMAN_attach failed (return code %d)\n",pname,pschedstat);
		exit(EXIT_FAILURE);
	}

	/*
	 * Variables
	 */
	FILE *fp;
	int n_pix, sstat, sem_atop_id, sem_ptot_id;
	char str[50];
	CVision objfinal, objfilter;
	HSVColour color;// = {30, 90, 70, 256, 100, 256};
	key_t sem_ptot_key, shmem_ptot_key;
	key_t sem_atop_key, shmem_atop_key;
	union dsemun sem_union;
	sem_ptot_key = SEM_PTOT_KEY;
	shmem_ptot_key = SHMEM_PTOT_KEY;
	sem_atop_key = SEM_ATOP_KEY;
	shmem_atop_key = SHMEM_ATOP_KEY;

/******************************
 * Retrieve semaphore ATOP ID *
 ******************************/
	sem_atop_id = semget((key_t)SEM_ATOP_KEY, 1, 0666);
	if(sem_atop_id == -1) 
		return -2;

/**********************************
 * Create and init semaphore PTOT *
 **********************************/
	sem_ptot_id = semget((key_t)SEM_PTOT_KEY, 1, 0666 | IPC_CREAT);
	if(sem_ptot_id == -1)
		return -2;

	sem_union.val=1;
	sstat=semctl(sem_ptot_id, 0, SETVAL, sem_union);
	if(sstat == -1) 
		return -2;

/************************************************************
 * Shared Memory for Acquisition to Processing Comunication *
 ************************************************************/
	void *shm_atop = (void *)0;
	CVision *sh_atop;
	int shm_atop_id;

	shm_atop_id = shmget(shmem_atop_key, sizeof(CVision), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_atop_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_ATOP on im_proc!\n");
		exit(EXIT_FAILURE);
	}
	shm_atop = shmat(shm_atop_id, (void *)0, 0);
	if(shm_atop == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_ATOP on im_proc!\n");
		exit(EXIT_FAILURE);
	}

	sh_atop = (CVision *)shm_atop;

/*********************************************************
 * Shared Memory for Processing to Tracking Comunication *
 *********************************************************/
	void *shm_ptot = (void *)0;
	struct PTOT *sh_ptot;
	int shm_ptot_id;

	shm_ptot_id = shmget(shmem_ptot_key, sizeof(struct PTOT), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_ptot_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_PTOT on im_proc!\n");
		exit(EXIT_FAILURE);
	}
	shm_ptot = shmat(shm_ptot_id, (void *)0, 0);
	if(shm_ptot == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_PTOT on im_proc!\n");
		exit(EXIT_FAILURE);
	}

	sh_ptot = (PTOT *)shm_ptot;
//****************************************************************************************
// Criação das imagens
//****************************************************************************************
	IplImage* pyr = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 3);//"PyrDown da Original";
	IplImage* hsv = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 3);//"Imagem no espaço HSV";
	//Componentes da imagem HSV
	IplImage* h = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);//"Componente H";
	IplImage* s = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);//"Componente S";
	IplImage* v = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);//"Componente V";

	IplImage *h_filter = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);
	IplImage *s_filter = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);
	IplImage *v_filter = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);

	IplImage* filter = cvCreateImage(cvSize(Gwidth/2,Gheight/2), Gdepth, 1);//"Filtro de Cor";
	IplImage* final = cvCreateImage(cvSize(Gwidth/2, Gheight/2), Gdepth, 3);

	/*
	* Image processing loop
	*/
//****************************************************************************************
// Leitura dos parametros armazenados
//****************************************************************************************
	if((fp = fopen("hsv_param.txt","r")) != NULL)
	{
		fscanf(fp,"HSV Color Detection parameters:\n",str);
		fscanf(fp,"H: %d\t%d\n",&(color.Hlow),&(color.Hhigh));
		fscanf(fp,"S: %d\t%d\n",&(color.Slow),&(color.Shigh));
		fscanf(fp,"V: %d\t%d\n",&(color.Vlow),&(color.Vhigh));
	}
	else
	{
		cout << "Could not open file!" << endl;
		exit(EXIT_FAILURE);
	}

	while(!end)
	{
		/* Wait for activation instant */
		sigsuspend(&set);

		PMAN_trace_evt_set(pname,'S');

		gettimeofday(&ta,NULL);

		printf("\n   [%s Starting time  %d, time:%6ld:%6ld]", argv[0],actcnt++,ta.tv_sec, ta.tv_usec);

//****************************************************************************************
// Determinação dos filtros a usar para a cor
//****************************************************************************************
// 		cvZero(pyr);
// 
		/* Critical Region 1 */
		sem_wait(sem_atop_id);
// 		cvPyrDown(sh_atop->m_pImg, pyr, CV_GAUSSIAN_5x5 );
		sh_atop->Display();
		sem_signal(sem_atop_id);
// 
// 		// Conversão para HSV
// 		cvZero(hsv);
// 		cvCvtColor(pyr, hsv, CV_BGR2HSV );
// 
// 		// Divisão em Componentes
// 		cvZero(h);cvZero(s);cvZero(v);
// 		cvSplit(hsv, h, s, v, NULL);
// 		cvZero(h_filter);
// 		cvInRangeS(h, cvScalar(color.Hlow), cvScalar(color.Hhigh), h_filter);
// 
// 		cvZero(s_filter);
// 		cvInRangeS(s, cvScalar(color.Slow), cvScalar(color.Shigh), s_filter);
// 
// 		cvZero(v_filter);
// 		cvInRangeS(v, cvScalar(color.Vlow), cvScalar(color.Vhigh), v_filter);	
// 
// 		cvZero(filter);
// 		cvAnd(h_filter, s_filter, filter, v_filter);
// 		// Aplica o filtro à imagem
// 		cvZero(final);
// 		cvCopy(pyr, final, filter);
// 
// 		n_pix = cvCountNonZero(filter);
// 		objfinal.m_pImg = final;
// 		objfilter.m_pImg = filter;
// 
// 		/* Critical Region 2 */
// 		sem_wait(sem_ptot_id);
// 		sh_ptot->n_pix = n_pix;
// 		sh_ptot->final = objfinal;
// 		sh_ptot->filter = objfilter;
// 		sem_signal(sem_ptot_id);

		PMAN_epilogue(pname); // Mark process as finished
	}

	/*
	* Release resources before exit
	*/

	// Release PtoT Shared Memory
	if(shmdt(shm_ptot) == -1)
	{
		fprintf(stderr, "shmdt failed dettaching SHM_PTOT on im_proc!\n");
		exit(EXIT_FAILURE);
	}

	// Release & Delete AtoP Shared Memory
	if(shmdt(shm_atop) == -1)
	{
		fprintf(stderr, "shmdt failed dettaching SHM_ATOP on im_proc!\n");
		exit(EXIT_FAILURE);
	}
	if(shmctl(shm_atop_id, IPC_RMID, 0) == -1)
	{
		fprintf(stderr, "shmctl failed deleting SHM_ATOP on im_proc!\n");
		exit(EXIT_FAILURE);
	}

	/* Deletes semaphore ATOP */
	sstat=semctl(sem_atop_id, 0, IPC_RMID, sem_union);
	if(sstat == -1)
	{
		fprintf(stderr,"semctl (IPC_RMID) failed while deleting SEM_ATOP!\n");
		return -3;
	}

	/* Close Parameter File */
	fclose(fp);

	// Release PMAN resources
	PMAN_close(PMAN_CLLEAVE);

	printf("\nCleanup complete!\n");

	exit(EXIT_SUCCESS);

}

void sighand(int sig)
{
  if(sig==SIGINT)
    end=1;
  return;
}
