/***************************************************************************
 *   Copyright (C) 2007 by Mauro Rodrigues				   *
 *   mauro.r@ua.pt							   *
 *                                                                         *
 *   PhUA - Projecto Humanóide da Universidade de Aveiro 07/08		   *
 *	@Sistemas de Tempo-Real 07/08					   *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <linux/types.h>
#include <math.h>
#include <pthread.h>

#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#include <sched.h>

#include <highgui.h>
#include <cv.h>
#include <cxcore.h>
#include <cvaux.h>

#include <pman.h>
#include <pmandefs.h>
#include <sem_utils.h>
#include "defs.h"
#include <visiondefs.h>

int end=0;

/* Prototypes */
void sighand(int sig);

/*
 *  Main
 */
int main(int argc, char *argv[]) 
{ 
/* **********************
 *	PMAN stuff 	*
 ************************/
	int pschedstat;            // Op status
	char *pname = "obj_track";// Process name (must be unique)
	key_t sem_psched_key,shmem_psched_key;
	int actcnt=0;         // Processe activation counter
	struct timeval ta;
	/* Signal management */
	sigset_t set;
	struct sigaction act;
	/*
	* Initializations
	*/
	/* Signal management set-up*/
	/* Sig mask with every signal but INT and PMAN_ACTIVATE_SIG */
	sigemptyset(&set);
	sigfillset(&set);
	sigdelset(&set,PMAN_ACTIVATE_SIG);
	sigdelset(&set,SIGINT);
	/* Signal handler */
	act.sa_handler=sighand;
	sigemptyset(&act.sa_mask);
	act.sa_flags=0;
	sigaction(PMAN_ACTIVATE_SIG,&act,0);
	sigaction(SIGINT,&act,0);
	/*
	* Initialize PMAN lib
	*/
	shmem_psched_key = SHMEM_PMAN_KEY;
	sem_psched_key   = SEM_PMAN_KEY;
	/* PMAN initializations (attach to existing process table)*/
	if((pschedstat = PMAN_init(SHMEM_PMAN_KEY, SEM_PMAN_KEY, NULL, 0,PMAN_ATTACH)))
	{
		fprintf(stderr, "\n [%s]: PMAN_init (PMAN_ATTACH) failed (return code %d)\n",pname, pschedstat);
		exit(EXIT_FAILURE);
	}
	/* Fill PID in process table */
	pschedstat=PMAN_attach(pname,getpid());
	if(pschedstat)
	{
		fprintf(stderr, "\n [%s]: PMAN_attach failed (return code %d)\n",pname,pschedstat);
		exit(EXIT_FAILURE);
	}

	/*
	 * Variables
	 */
	int sstat, sem_ptot_id, sem_rtdb_id;
	//char ball[]="ball\0";
	double M00,M10,M01;
	CvPoint obj_cen, cen, ang, cir_cen;
	CvMoments moments;
	cen.x = Gwidth/2;
	cen.y = Gheight/2;
	obj_cen = cen;
	key_t sem_ptot_key, shmem_ptot_key;
	key_t sem_rtdb_key, shmem_rtdb_key;
	union dsemun sem_union;
	sem_ptot_key = SEM_PTOT_KEY;
	shmem_ptot_key = SHMEM_PTOT_KEY;
	sem_rtdb_key = SEM_RTDB_KEY;
	shmem_rtdb_key = SHMEM_RTDB_KEY;

/******************************
 * Retrieve semaphore PTOT ID *
 ******************************/
	sem_ptot_id = semget((key_t)SEM_PTOT_KEY, 1, 0666);
	if(sem_ptot_id == -1) 
		return -2;

/**********************************
 * Create and init semaphore RTDB *
 **********************************/
	sem_rtdb_id = semget((key_t)SEM_RTDB_KEY, 1, 0666 | IPC_CREAT);
	if(sem_rtdb_id == -1)
		return -2;

	sem_union.val=1;
	sstat=semctl(sem_rtdb_id, 0, SETVAL, sem_union);
	if(sstat == -1) 
		return -2;

/************************************************************
 * Shared Memory for Processing to Tracking Comunication *
 ************************************************************/
	void *shm_ptot = (void *)0;
	struct PTOT *sh_ptot;
	int shm_ptot_id;

	shm_ptot_id = shmget(shmem_ptot_key, sizeof(struct PTOT), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_ptot_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_PTOT on im_proc!\n");
		exit(EXIT_FAILURE);
	}
	shm_ptot = shmat(shm_ptot_id, (void *)0, 0);
	if(shm_ptot == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_PTOT on im_proc!\n");
		exit(EXIT_FAILURE);
	}

	sh_ptot = (PTOT *)shm_ptot;

/****************************************
 * Shared Memory for Real Time DataBase *
 ****************************************/
	void *shm_rtdb = (void *)0;
	struct RTDB *sh_rtdb;
	int shm_rtdb_id;

	shm_rtdb_id = shmget(shmem_rtdb_key, sizeof(RTDB), 0666 | IPC_CREAT);
	/* Create and attach Shared Memory to Process */
	if(shm_rtdb_id == -1)
	{
		fprintf(stderr, "shmget failed creating SHM_RTDB on obj_track!\n");
		exit(EXIT_FAILURE);
	}
	shm_rtdb = shmat(shm_rtdb_id, (void *)0, 0);
	if(shm_rtdb == (void *)-1)
	{
		fprintf(stderr, "shmat failed attaching SHM_RTDB on obj_track!\n");
		exit(EXIT_FAILURE);
	}

	sh_rtdb = (RTDB *)shm_rtdb;

	IplImage* filter = cvCreateImage(cvSize(Gwidth/2, Gheight/2), Gdepth, 3);
	IplImage* final = cvCreateImage(cvSize(Gwidth/2, Gheight/2), Gdepth, 3);
	IplImage* final_gray= cvCreateImage(cvSize(Gwidth/2, Gheight/2), Gdepth, 1);
	/*
	* Image processing loop
	*/
	while(!end)
	{
		/* Wait for activation instant */
		sigsuspend(&set);

		PMAN_trace_evt_set(pname,'S');

		gettimeofday(&ta,NULL);

		printf("\n   [%s Starting time  %d, time:%6ld:%6ld]", argv[0],actcnt++,ta.tv_sec, ta.tv_usec);

		/* Critical Region*/
		sem_wait(sem_ptot_id);
//****************************************************************************************
// Inicio do Tracking do Objecto
//****************************************************************************************
		if(sh_ptot->n_pix > 50)
		{
//****************************************************************************************
// Calculo do centro de massa
//****************************************************************************************

			/* Access to Critical Region - Creation of local copies */
			cvCopy(sh_ptot->filter.m_pImg, filter, NULL);
			cvCopy(sh_ptot->final.m_pImg, final, NULL);
			sem_signal(sem_ptot_id);

			cvMoments(sh_ptot->filter.m_pImg, &moments, 1 );
			M00=cvGetSpatialMoment( &moments, 0, 0 );
			M10=cvGetSpatialMoment( &moments, 1, 0 );
			M01=cvGetSpatialMoment( &moments, 0, 1 );
			if ((((int) (M10/M00)) < Gwidth/2) && (((int) (M10/M00)) > 0))
				obj_cen.x= ((int) (M10/M00));
			if((((int) (M01/M00)) < Gheight/2) && (((int) (M01/M00)) > 0))
				obj_cen.y= ((int) (M01/M00));

//****************************************************************************************
// Verificação do circulo
//****************************************************************************************
			CvMemStorage* storage = cvCreateMemStorage(0);
			cvCvtColor(final, final_gray, CV_BGR2GRAY );
			cvSmooth(final_gray, final_gray, CV_GAUSSIAN, 1, 1 ,0); // smooth it, otherwise a lot of false circles may be detected
			CvSeq* circles = cvHoughCircles(final_gray, storage, CV_HOUGH_GRADIENT, final_gray->height/20,500, 10,1 );
			float* p=NULL;
			char* elem;

			if (( elem = cvGetSeqElem( circles, 1)) > 0)
			{
		 		p = (float*) elem;
        		}
			if(((int)cvRound(p[2]))>10 && ((int) cvRound(p[2]))<100)
			{
				cvCircle(final, cvPoint((int) cvRound(p[0]),(int) cvRound(p[1])),(int) cvRound(p[2]), CV_RGB(128,0,0), 3, 8, 0 );
       			}

			cir_cen.x=cvRound(p[0]);
			cir_cen.y=cvRound(p[1]);

			static int j=0;

			if(cvRound(p[2])>10 && cvRound(p[2])<100)
			{
				if(obj_cen.x-cvRound(p[0])<35)
				{
					if(obj_cen.y-cvRound(p[1])<35)
					{
						j++;
						if(j==1)
						{	
							j=0;
							ang.x=int(-(obj_cen.x-cen.x)/6);
							ang.y=int(-(obj_cen.y-cen.y)/6);
						}
					}
				}
			}
			/* Critical Region 2 */
			sem_wait(sem_rtdb_id);
			sh_rtdb->xpos = obj_cen.x;
			sh_rtdb->ypos = obj_cen.y;
			sh_rtdb->xerr = Gwidth/2-obj_cen.x;
			sh_rtdb->yerr = Gheight/2-obj_cen.y;
			sem_signal(sem_rtdb_id);
		}
		else
			sem_signal(sem_ptot_id);

		PMAN_epilogue(pname); // Mark process as finished

	}

	/*
	* Release resources before exit
	*/
	// Release RTDB Shared Memory
	if(shmdt(shm_rtdb) == -1)
	{
		fprintf(stderr, "shmdt failed dettaching RTDB on obj_track!\n");
		exit(EXIT_FAILURE);
	}

	// Release & Delete PtoT Shared Memory
	if(shmdt(shm_ptot) == -1)
	{
		fprintf(stderr, "shmdt failed dettaching SHM_ATOP on obj_track!\n");
		exit(EXIT_FAILURE);
	}
	if(shmctl(shm_ptot_id, IPC_RMID, 0) == -1)
	{
		fprintf(stderr, "shmctl failed deleting SHM_PTOT on obj_track!\n");
		exit(EXIT_FAILURE);
	}

	/* Deletes semaphore PTOT */
	sstat=semctl(sem_ptot_id, 0, IPC_RMID, sem_union);
	if(sstat == -1)
	{
		fprintf(stderr,"semctl (IPC_RMID) failed while deleting SEM_PTOT!\n");
		return -3;
	}

	// Release PMAN resources
	PMAN_close(PMAN_CLLEAVE);

	printf("\nCleanup complete!\n");
	exit(EXIT_SUCCESS);

}

void sighand(int sig)
{
  if(sig==SIGINT)
    end=1;
  return;
}
